import chat.Failure;

/**
 * Classe abstraite de base pour un client ou un serveur de chat
 * @author davidroussel
 */
public abstract class AbstractRunChat
{
	/**
	 * Port  utiliser pour les connnections entre clients et serveur
	 * @uml.property  name="port"
	 */
	protected int port;

	/**
	 * numero de port de communication par dfaut
	 */
	public static final int DEFAULTPORT = 1394;

	/**
	 * Etat de verbose. Si true les messages de debug seront affichs. Si false les messages de debug ne seront pas affichs
	 * @uml.property  name="verbose"
	 */
	protected boolean verbose;

	/**
	 * Constructeur d'un client ou d'un serveur de chat d'aprs les arguments
	 * fournis au programme principal
	 * @param args les arguments fournis au programme principal en vue de
	 * mettre en place certaines options particulire  un client ou un serveur
	 * Recherche des valeurs pour {@link #port} et {@link #verbose} dans les
	 * chanes de caractres fournis en arguments
	 */
	protected AbstractRunChat(String[] args)
	{
		setAttributes(args);
	}

	/**
	 * Mise en place des valeurs des attributs et parsing des arguments
	 * @param args les arguments fournis au programme principal en vue de
	 * mettre en place certaines options particulire  un client ou un serveur
	 * Recherche des valeurs pour {@link #port} et {@link #verbose} dans les
	 * chanes de caractres fournis en arguments
	 */
	protected void setAttributes(String[] args)
	{
		/*
		 * On met d'abord les attributs locaux  leur valeur par dfaut
		 */
		port = DEFAULTPORT;
		verbose = false;

		/*
		 * parsing des arguments
		 * 	-v | --verbose
		 * 	-p | --port : port  utiliser pour la serverSocket
		 */
		for (int i=0; i < args.length; i++)
		{
			if (args[i].startsWith("-")) // option argument
			{
				if (args[i].equals("--verbose") || args[i].equals("-v"))
				{
					System.out.println("Setting verbose on");
					verbose = true;
				}
				else if (args[i].equals("--port") || args[i].equals("-p"))
				{
					System.out.print("Setting port to : ");
					if (i < (args.length - 1))
					{
						// parse next arg for in port value
						Integer portInteger = readInt(args[++i]);
						if (portInteger != null)
						{
							int readPort = portInteger.intValue();
							if (readPort >= 1024)
							{
								port = readPort;
							}
							else
							{
								System.err.println(Failure.INVALID_PORT);
								System.exit(Failure.INVALID_PORT.toInteger());
							}
						}
						System.out.println(port);
					}
					else
					{
						System.out.println("nothing, invalid value");
					}
				}
			}
		}
	}

	/**
	 * Une fois le client ou le serveur prt, on lance son excution
	 */
	protected abstract void launch();

	/**
	 * Lecture d'un entier  partir d'une chane de caractres
	 * @param s la chaine  lire
	 * @return l'entier pars dans la chaine de caractre ou bien null
	 * s'il s'est produit une erreur de parsing
	 */
	protected static Integer readInt(String s)
	{
		try
		{
			Integer value = new Integer(Integer.parseInt(s));
			return value;
		}
		catch (NumberFormatException e)
		{
			System.err.println("readInt : " + s + " is not a number");
			return null;
		}
	}
}
