import java.io.IOException;
import java.net.SocketException;

import chat.Failure;
import chat.server.ChatServer;

/**
 * Classe/programme qui lance un serveur de chat
 * @author davidroussel
 *
 */
public class RunChatServer extends AbstractRunChat
{
	/**
	 * Time out de la server socket avant qu'elle ne recommence  attendre des connections des ventuels clients
	 * @uml.property  name="timeout"
	 */
	private int timeout;

	/**
	 * Default time out to wait for client connection : 5 seconds
	 */
	public static final int DEFAULTTIMEOUT = 5000;

	/**
	 * Constructeur d'un lanceur de serveur d'aprs les arguments du programme
	 * principal
	 * @param args les arguments du programme principal
	 */
	protected RunChatServer(String[] args)
	{
		super(args);
	}

	@Override
	protected void setAttributes(String[] args)
	{
		/*
		 * parsing des arguments communs aux clients et serveur
		 * 	-v | --verbose
		 * 	-p | --port : port  utiliser pour la serverSocket
		 */
		super.setAttributes(args);

		/*
		 * On met d'abord les attributs locaux  leur valeur par dfaut
		 */
		timeout = DEFAULTTIMEOUT;

		/*
		 * parsing des arguments spcifique au  serveur
		 * 	-t | --timeout : timeout d'attente de la server socket
		 */
		for (int i=0; i < args.length; i++)
		{
			if (args[i].equals("--timeout") || args[i].equals("-t"))
			{
				System.out.print("Setting timeout to : ");
				if (i < (args.length - 1))
				{
					// parse next arg for in port value
					Integer timeInteger = readInt(args[++i]);
					if (timeInteger != null)
					{
						timeout = timeInteger.intValue();
					}
					System.out.println(timeout);
				}
				else
				{
					System.out.println("nothing, invalid value");
				}
			}
		}
	}

	@Override
	protected void launch()
	{
		/*
		 * Create and Launch server on local ip adress with port number and
		 * verbose status
		 */
		System.out.println("Creating server on port " + port
				+ " with timeout " + timeout + " ms and verbose "
				+ (verbose ? "on" : "off"));
		ChatServer server = null;
		try
		{
			server = new ChatServer(port, verbose, timeout);
		}
		catch (SocketException se)
		{
			System.err.println(Failure.SET_SERVER_SOCKET_TIMEOUT + ", abort ...");
			se.printStackTrace();
			System.exit(Failure.SET_SERVER_SOCKET_TIMEOUT.toInteger());
		}
		catch (IOException e)
		{
			System.err.println(Failure.CREATE_SERVER_SOCKET + ", abort ...");
			e.printStackTrace();
			System.exit(Failure.CREATE_SERVER_SOCKET.toInteger());
		}

		Thread serverThread = null;
		if (server != null)
		{
			System.out.print("Launching server ... ");
			serverThread = new Thread(server);
			serverThread.start();
			System.out.println("done");

			if (serverThread != null)
			{
				if (verbose)
				{
					System.err.print("Waiting for server to terminate ... ");
				}
				try
				{
					/*
					 * On suspend le thread courant dans l'attente de la fin
					 * de l'excution de serverThread
					 */
					serverThread.join();
					if (verbose)
					{
						System.err.println("Server terminated, program end.");
					}
				}
				catch (InterruptedException e)
				{
					System.err.println("Server Thread Join interrupted");
					e.printStackTrace();
				}
			}
		}
	}

	/**
	 * Programme principal
	 * @param args les arguments
	 * <ul>
	 * 	<li>--port <port number> : set host connection port</li>
	 * 	<li>--verbose : set verbose on</li>
	 * 	<li>--timeout <timeout in ms> : server socket waiting time out</li>
	 * </ul>
	 */
	public static void main(String[] args)
	{
		RunChatServer server = new RunChatServer(args);

		server.launch();
	}
}
