import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;

/**
 * Exemple de classe implmentant un Runnable et lanc dans un Thread
 *
 * @author davidroussel
 */
public class RunnableExample
{
	protected static class Counter implements Runnable
	{
		protected int count;

		protected int max;

		public Counter(int max)
		{
			count = 0;
			this.max = max;
		}

		@Override
		public void run()
		{
			while (count < max)
			{
				count++;
				System.out.println(this + " count = " + count);

				// passe la main  d'autres threads (si besoin)
				Thread.yield();
			}
		}
	}

	/**
	 * @uml.property  name="counters"
	 * @uml.associationEnd  multiplicity="(0 -1)" elementType="RunnableExample$Counter"
	 */
	protected Collection<Counter> counters;

	/**
	 * @uml.property  name="threads"
	 * @uml.associationEnd  multiplicity="(0 -1)" elementType="java.lang.Thread"
	 */
	protected Collection<Thread> threads;

	/**
	 * Constructeur d'un RunnableExample. Cre deux Counter
	 */
	public RunnableExample(int nbCounters)
	{
		counters = new ArrayList<Counter>(nbCounters);
		threads = new ArrayList<Thread>(nbCounters);

		for (int i = 0; i < nbCounters; i++)
		{
			Counter c = new Counter(10);
			counters.add(c);

			Thread t = new Thread(c);
			threads.add(t);
		}
	}

	/**
	 * Lancement de tous les threads
	 */
	public void launch()
	{
		for (Thread t : threads)
		{
			t.start();
		}
	}

	/**
	 * attente de la fin de tous les threads pour terminer le thread principal
	 */
	public void terminate()
	{
		for (Thread t : threads)
		{
			try
			{
				t.join();
			}
			catch (InterruptedException e)
			{
				System.err.println("Thread" + t + " join interrupted");
				e.printStackTrace();
			}
		}

		System.out.println("All threads terminated");
	}

	/**
	 * Programme principal.
	 * Lancement de plusieurs Counters
	 *
	 * @param args arguments du programme
	 */
	public static void main(String[] args)
	{
		int nbCounters = 3;
		// on lit le nombre de counters dans le premier argument du programme
		if (args.length > 0)
		{
			int value;
			try
			{
				value = Integer.parseInt(args[0]);
				if (value > 0)
				{
					nbCounters = value;
				}
			}
			catch (NumberFormatException nfe)
			{
				System.err.println("Error reading number of counters");
			}
		}

		RunnableExample runner = new RunnableExample(nbCounters);

		runner.launch();
		System.out.println("All threads launched");

		runner.terminate();
		System.out.println("program end, press Enter to terminate.");
		try
		{
			System.in.read();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
}
