package chat;

/**
 * Enumration de toutes les erreurs possibles dans le systme client/serveur de
 * chat.
 * 
 * @author davidroussel
 */
public enum Failure
{
	/**
	 * @uml.property name="nO_LOCAL_HOST"
	 * @uml.associationEnd
	 */
	NO_LOCAL_HOST,
	/**
	 * @uml.property name="iNVALID_PORT"
	 * @uml.associationEnd
	 */
	INVALID_PORT,
	/**
	 * @uml.property name="nO_USER_NAME"
	 * @uml.associationEnd
	 */
	NO_USER_NAME,
	/**
	 * @uml.property name="nO_ENV_ACCESS"
	 * @uml.associationEnd
	 */
	NO_ENV_ACCESS,
	/**
	 * @uml.property name="sET_SERVER_SOCKET_TIMEOUT"
	 * @uml.associationEnd
	 */
	SET_SERVER_SOCKET_TIMEOUT,
	/**
	 * @uml.property name="cREATE_SERVER_SOCKET"
	 * @uml.associationEnd
	 */
	CREATE_SERVER_SOCKET,
	/**
	 * @uml.property name="uNKNOWN_HOST"
	 * @uml.associationEnd
	 */
	UNKNOWN_HOST,
	/**
	 * @uml.property name="cLIENT_CONNECTION"
	 * @uml.associationEnd
	 */
	CLIENT_CONNECTION,
	/**
	 * @uml.property name="cLIENT_INPUT_STREAM"
	 * @uml.associationEnd
	 */
	CLIENT_INPUT_STREAM,
	/**
	 * @uml.property name="cLIENT_OUTPUT_STREAM"
	 * @uml.associationEnd
	 */
	CLIENT_OUTPUT_STREAM,
	/**
	 * @uml.property name="uSER_INPUT_STREAM"
	 * @uml.associationEnd
	 */
	USER_INPUT_STREAM,
	/**
	 * @uml.property name="uSER_OUTPUT_STREAM"
	 * @uml.associationEnd
	 */
	USER_OUTPUT_STREAM,
	/**
	 * @uml.property name="sERVER_CONNECTION"
	 * @uml.associationEnd
	 */
	SERVER_CONNECTION,
	/**
	 * @uml.property name="sERVER_INPUT_STREAM"
	 * @uml.associationEnd
	 */
	SERVER_INPUT_STREAM,
	/**
	 * @uml.property name="sERVER_OUTPUT_STREAM"
	 * @uml.associationEnd
	 */
	SERVER_OUTPUT_STREAM,
	/**
	 * @uml.property name="nO_NAME_CLIENT"
	 * @uml.associationEnd
	 */
	NO_NAME_CLIENT,
	/**
	 * @uml.property name="cLIENT_NOT_READY"
	 * @uml.associationEnd
	 */
	CLIENT_NOT_READY,
	/**
	 * @uml.property name="oTHER"
	 * @uml.associationEnd
	 */
	OTHER;

	/**
	 * Affichage sous forme de texte des erreurs possibles
	 */
	@Override
	public String toString()
	{
		switch (this)
		{
		// RunChatClient Failures (3)
			case NO_LOCAL_HOST:
				return new String("Unable to get local host name");
			case INVALID_PORT:
				return new String("Port number should be > 1024");
			case NO_USER_NAME:
				return new String("Empty user name");
			case NO_ENV_ACCESS:
				return new String(
						"System does not allow access to environment variables");
				// RunChatServer Failures (2)
			case SET_SERVER_SOCKET_TIMEOUT:
				return new String("Unable to set Server socket timeout");
			case CREATE_SERVER_SOCKET:
				return new String("Unable to create Server socket");
				// Chat Client (4)
			case UNKNOWN_HOST:
				return new String("Unkown host");
			case CLIENT_CONNECTION:
				return new String("Couldn't get I/O for connection to host");
			case CLIENT_INPUT_STREAM:
				return new String("Could not get intput stream from client");
			case CLIENT_OUTPUT_STREAM:
				return new String("Could not get output stream to client");
				// ServerHandler (2)
			case USER_INPUT_STREAM:
				return new String("Could not get input stream from user");
				// ServerHandler
			case USER_OUTPUT_STREAM:
				return new String("Could not get output stream to user");
				// ChatServer#run (3)
			case SERVER_CONNECTION:
				return new String("Client connection to server failed");
			case SERVER_INPUT_STREAM:
				return new String("could not get input stream from server");
			case SERVER_OUTPUT_STREAM:
				return new String("could not get output stream to server");
			case NO_NAME_CLIENT:
				return new String("Unable to read client's name");
				// Client (1)
			case CLIENT_NOT_READY:
				return new String("Main Client not ready");
			case OTHER:
				return new String("Other cause");
		}
		throw new AssertionError("Failure : unknown op : " + this);
	}

	/**
	 * Conversion en entier du type d'erreur
	 * 
	 * @return le numro de l'erreur
	 * @code System.exit(Failure.CLIENT_NOT_READY.toInteger());
	 * @endcode
	 */
	public int toInteger()
	{
		return ordinal() + 1;
	}

}
