package chat.server;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.Socket;

import chat.Failure;


/**
 * Classe stockant les caractristiques d'un client :
 * voir {@link MainClient}.
 * Un client "normal" ajoute aussi le flux de sortie sur lequel on crit vers
 * le client
 * <ul>
 * 	<li>out : {@link PrintWriter}</li>
 * </ul>
 * @author davidroussel
 *
 */
public class Client extends MainClient
{
	/**
	 * Le flux de sortie du client (celui sur lequel on crit au client)
	 * @uml.property  name="outPW"
	 */
	private PrintWriter outPW;

	/**
	 * Constructeur d'un client
	 * @param socket la socket du client
	 * @param name le nom du client
	 * @param verbose niveau de debug pour les messages
	 */
	public Client(Socket socket, String name, boolean verbose)
	{
		super(socket, name, verbose);
		if (ready)
		{
			outPW = null;
			ready = false;

			if (clientSocket != null)
			{
				if (verbose)
				{
					System.err.print("Client : Creating Output Stream ... ");
				}
				try
				{
					outPW = new PrintWriter(clientSocket.getOutputStream(), true);
					if (verbose)
					{
						System.err.println("done");
					}
					ready = true;
				}
				catch (IOException e)
				{
					System.err.println("Client : unable to get client output stream");
					e.printStackTrace();
				}
			}
		}
		else
		{
			System.err.println("Client: " + Failure.CLIENT_NOT_READY + ", abort...");
			System.exit(Failure.CLIENT_NOT_READY.toInteger());
		}
	}

	/**
	 * Accesseur en lecture du flux de sortie d'un client
	 * @return the out
	 */
	public PrintWriter getOut()
	{
		return outPW;
	}

	/**
	 * Nettoyage d'un client : fermeture du flux de sortie et super.cleanup()
	 */
	@Override
	public void cleanup()
	{
		if (verbose)
		{
			System.err.print("Client::cleanup : closing output stream ... ");
		}
		outPW.close();
		if (verbose)
		{
			System.err.println("done");
		}
		super.cleanup();
	}


}
