package widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import widgets.listeners.ClearListener;
import chat.Vocabulary;

/**
 * Exemple simple de fentre graphique
 * @author davidroussel
 */
public class ChatGraphique extends JFrame
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * @uml.property  name="btnLeftButton"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected JButton btnLeftButton;
	
	protected JLabel serveur;
	protected JTextField message;
	protected JButton send;
	protected JScrollPane chat;
	
	/**
	 * @uml.property  name="btnClearButton"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected JButton btnClearButton;

	/**
	 * @uml.property  name="document"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected StyledDocument document;

	/**
	 * @uml.property  name="style"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected Style style;

	/**
	 * @uml.property  name="defaultColor"
	 */
	protected Color defaultColor;

	/**
	 * Cration d'une fentre graphique simple
	 * @throws HeadlessException
	 */
	public ChatGraphique() throws HeadlessException
	{
		setPreferredSize(new Dimension(400, 200));

		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		getContentPane().add(toolBar, BorderLayout.NORTH);

		btnLeftButton = new JButton("Se deconnecter");
		toolBar.add(btnLeftButton);

	
		btnClearButton = new JButton("Clear");
		toolBar.add(btnClearButton);

		Component horizontalGlue = Box.createHorizontalGlue();
		toolBar.add(horizontalGlue);

		
		serveur = new JLabel("Serveur : 127.0.0.1:0000");
		toolBar.add(serveur);

		chat = new JScrollPane();
		getContentPane().add(chat, BorderLayout.CENTER);

		JTextPane textPane = new JTextPane();
		textPane.setEditable(false);

		chat.setViewportView(textPane);

		
		JToolBar toolBar2 = new JToolBar();
		getContentPane().add(toolBar2, BorderLayout.SOUTH);
		toolBar2.setFloatable(false);
		
		
		message = new JTextField(8);
        message.setLocation(0, 0);
        message.setSize(100, 30);
		
		toolBar2.add(message);

		toolBar2.add(Box.createHorizontalGlue());
		
		send  = new JButton("Send button");
		toolBar2.add(send);
		document = textPane.getStyledDocument();
		style = textPane.addStyle("New Style", null);
		defaultColor = StyleConstants.getForeground(style);

		/*
		 * ActionListener anonyme pour le btnLeftButton
		 */
		btnLeftButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				/*
				 * Change la couleur du texte en rouge et affiche un message
				 */
				addMessage("red", Color.RED);

				StyleConstants.setForeground(style, defaultColor);
			}
		});

		/*
		 * ActionListener externe pour le btnClearButton
		 */
		btnClearButton.addActionListener(new ClearListener(document));

	}

	/**
	 * Ajout d'un texte  la fin du document avec la couleur voulue
	 * @param text le texte  ajouter
	 * @param color la couleur du texte
	 */
	protected void addMessage(String text, Color color)
	{
		StyleConstants.setForeground(style, color);

		try
		{
			document.insertString(document.getLength(), text
					+ Vocabulary.newLine, style);
		}
		catch (BadLocationException ex)
		{
			System.err.println("write at bad location");
			ex.printStackTrace();
		}

	}

	/**
	 * Action listener interne  la classe ExampleFrame
	 */
	protected class RightButtonListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			/*
			 * RightButtonListener tant une classe interne (non static) elle a
			 * donc accs aux attributs de la classe ExampleFrame
			 * Change la couleur du texte en bleu et affiche un message
			 */
			addMessage("blue", Color.BLUE);

			StyleConstants.setForeground(style, defaultColor);
		}
	}
}
