package widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import widgets.listeners.ClearListener;

import chat.Vocabulary;

/**
 * FenÍtre d'affichae de la version GUI du client de chat.
 *
 * @author davidroussel
 */
public class ClientFrame extends JFrame implements Runnable
{
	protected JButton btnLeftButton; //Bouton de déconnexion
	protected JLabel serveur; //Affichage des informations sur le serveur 
	protected JTextField message; //Zone de texte pour insérer notre message 
	protected JButton send; //Bouton pour envoyer le message 
	protected JButton btnClearButton; //Bouton pour effacer les anciens messages 
	protected JScrollPane chat;// Conteneur pour l'affichage du chat
	protected JTextPane textPane; // Affichage du chat
	
	protected StyledDocument document; //Style pour le document
	protected Style style;//Style pour le document
	
	protected PipedOutputStream out; //Sortie std
	protected PipedInputStream in; // Entrée std

	protected OutputStreamWriter osw; //Permet de traiter la sortie
	protected BufferedWriter bw; //Permet de traiter la sortie
	
	protected String name; //Nom du client
	
	/**
	 * Constructeur de la fenÍtre
	 *
	 * @param name le nom de l'utilisateur
	 * @param host l'hÙte sur lequel on est connectÈ
	 * @param commonRun Ètat d'exÈcution des autres threads du client
	 * @param verbose pour les messages de debug
	 * @throws HeadlessException
	 */
	public ClientFrame(String name, String host, Boolean commonRun,
			boolean verbose) throws HeadlessException
	{
		
		//INITIALISATION DES DONNÉES
		//Nous enregistrons le nom du client
		this.name = name;
		
		//Instanciation des flux
		out = new PipedOutputStream();
		in = new PipedInputStream();

		//Le flux de sortie doit être traité dans un buffer
		osw = new OutputStreamWriter(out);
		bw = new BufferedWriter(osw);
		
		//Affichage de la fenêtre + dimensions
		this.setVisible(true);
		this.setPreferredSize(new Dimension(400, 200));
		this.setSize(new Dimension(400, 200));

		//CONSTRUCTION DE LA FENÊTRE
		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		getContentPane().add(toolBar, BorderLayout.NORTH);

		btnLeftButton = new JButton("Se deconnecter");
		toolBar.add(btnLeftButton);

		btnClearButton = new JButton("Clear");
		toolBar.add(btnClearButton);

		//Espace entre cler+déco et paramêtres du serveur
		Component horizontalGlue = Box.createHorizontalGlue();
		toolBar.add(horizontalGlue);

		//Affichage des informations serveur
		serveur = new JLabel("Serveur : "+host);
		toolBar.add(serveur);

		chat = new JScrollPane();
		getContentPane().add(chat, BorderLayout.CENTER);

		textPane = new JTextPane();
		textPane.setEditable(false);
		document = textPane.getStyledDocument();

		chat.setViewportView(textPane);

		//Berre en dessous pour le message + envoi.
		JToolBar toolBar2 = new JToolBar();
		getContentPane().add(toolBar2, BorderLayout.SOUTH);
		toolBar2.setFloatable(false);
		
		message = new JTextField(8);
        message.setLocation(0, 0);
        message.setSize(100, 30);
		
		toolBar2.add(message);

		toolBar2.add(Box.createHorizontalGlue());
		
		send  = new JButton("Send button");
		send.addActionListener(new SendButtonListener());
		toolBar2.add(send);
		
		style = textPane.addStyle("New Style", null);
		
		//Ajout des listenners
		btnClearButton.addActionListener(new ClearListener(document));
		btnLeftButton.addActionListener(new QuitServer());
	}

	//Accesseurs sur les flux
	public PipedOutputStream getOut() {
        return out;
    }

	public PipedInputStream getIn() {
		return in;
	}

	/**
	 * ExÈcution de la boucle d'exÈcution. La boucle d'exÈcution consiste ‡ lire
	 * une ligne sur le flux d'entrÈe avec un BufferedReader tant qu'une erreur
	 * d'IO n'intervient pas indiquant que le flux a ÈtÈ coupÈ. Puis afficher
	 * ce message datÈ dans le document sous-jacent du textPane.
	 */
	@Override
	public void run()
	{	
		//Initialisation en local des flux d'entrée
		InputStreamReader isr = new InputStreamReader(in);
		BufferedReader br = new BufferedReader(isr);
		
		//Lecture du buffer
		String ligne;
		try {
			while ((ligne=br.readLine())!=null){
				//Affichage du message
				addMessage(ligne, generateColor(name));
			}
		} catch (IOException e) {
			e.printStackTrace();
		}

		cleanup();
	}

	/**
	 * Fermeture de la fenËtre et des flux ‡ la fin de l'exÈcution
	 */
	public void cleanup()
	{
		
		try {
			//Fermeture des flux
			bw.close();
			osw.close();
			out.close();
			in.close();
		} catch (IOException e) {
			
			e.printStackTrace();
	
		}
	}
	
	/**
	 * Ajout d'un texte ‡ la fin du document avec la couleur voulue
	 * @param text le texte ‡ ajouter
	 * @param color la couleur du texte
	 */
	protected void addMessage(String text, Color color)
	{
		StyleConstants.setForeground(style, color);

		try
		{
			//La date
			DateFormat df = new SimpleDateFormat("yyyy/MM/dd");
			String formattedDate = df.format(new Date()); 
			
			document.insertString(document.getLength(),
					"["+formattedDate+"] "+text
					+ Vocabulary.newLine, style);
		}
		catch (BadLocationException ex)
		{
			System.err.println("write at bad location");
			ex.printStackTrace();
		}

	}
	
	protected void sendMessage(String msg)
	{
		try {
			
			bw.write(msg); //Ecriture dans le flux de sortie
			bw.newLine();//Nouvelle ligne
			bw.flush();//Envoi
			
			//bw.close();
			
		} catch (IOException e) {
		
			e.printStackTrace();
		}
		
		
	}
	
	/*
	 * Gestion des couleurs
	 */
	protected Color generateColor(String name)
	{
		Color c=null;
		switch(name.hashCode()%8)
		{
			case 0 : 	c=Color.BLACK ; break;
			case 1 : 	c=Color.BLUE ; break;
			case 2 : 	c=Color.GRAY ; break;
			case 3 : 	c=Color.GREEN ; break;
			case 4 : 	c=Color.MAGENTA ; break;
			case 5 : 	c=Color.ORANGE ; break;
			case 6 : 	c=Color.PINK ; break;
			case 7 : 	c=Color.RED ; break;
		}
		
		return c;
	}
	
	protected class SendButtonListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			if (!message.getText().equals(""))
			{
				sendMessage(message.getText());
				message.setText("");
			}
			
		}
	}
	
	protected class QuitServer implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			sendMessage("bye");
			
		}
	}
}
