/**
 *
 */
package widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.JToolBar;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import widgets.listeners.ClearListener;

/**
 * @author davidroussel
 */
public class FrameExample extends JFrame
{

	/**
	 * Sauts de ligne du systme d'exploitation (utilis dans le texte)
	 */
	public final static String newline = System.getProperty("line.separator");

	/**
	 * @uml.property  name="btnLeftButton"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected JButton btnLeftButton;

	/**
	 * @uml.property  name="btnRightButton"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected JButton btnRightButton;

	/**
	 * @uml.property  name="btnClearButton"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected JButton btnClearButton;

	/**
	 * @uml.property  name="document"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected StyledDocument document;

	/**
	 * @uml.property  name="style"
	 * @uml.associationEnd  multiplicity="(1 1)"
	 */
	protected Style style;

	/**
	 * @uml.property  name="defaultColor"
	 */
	protected Color defaultColor;

	/**
	 * @throws HeadlessException
	 */
	public FrameExample() throws HeadlessException
	{

		JToolBar toolBar = new JToolBar();
		toolBar.setFloatable(false);
		getContentPane().add(toolBar, BorderLayout.NORTH);

		btnLeftButton = new JButton("Red");
		toolBar.add(btnLeftButton);

		btnRightButton = new JButton("Blue");
		toolBar.add(btnRightButton);

		Component horizontalGlue = Box.createHorizontalGlue();
		toolBar.add(horizontalGlue);

		btnClearButton = new JButton("Clear");
		toolBar.add(btnClearButton);

		JScrollPane scrollPane = new JScrollPane();
		getContentPane().add(scrollPane, BorderLayout.CENTER);

		JTextPane textPane = new JTextPane();
		scrollPane.setViewportView(textPane);

		document = textPane.getStyledDocument();
		style = textPane.addStyle("New Style", null);
		defaultColor = StyleConstants.getForeground(style);

		/*
		 * ActionListener anonyme pour le btnLeftButton
		 */
		btnLeftButton.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent e)
			{
				/*
				 * Change la couleur du texte en rouge et affiche un message
				 */
				StyleConstants.setForeground(style, Color.RED);
				try
				{
					document.insertString(document.getLength(),
							"Red" + newline, style);
				}
				catch (BadLocationException ex)
				{
					System.err.println("write at bad location");
					ex.printStackTrace();
				}
				StyleConstants.setForeground(style, defaultColor);
			}
		});

		/*
		 * ActionListener interne pour le btnRightButton
		 */
		btnRightButton.addActionListener(new RightButtonListener());

		/*
		 * ActionListener externe pour le btnClearButton
		 */
		btnClearButton.addActionListener(new ClearListener(document));

	}

	/**
	 * Action listener interne  la classe FrameExample
	 */
	protected class RightButtonListener implements ActionListener
	{
		@Override
		public void actionPerformed(ActionEvent e)
		{
			/*
			 * RightButtonListener tant une classe interne (non static) elle a
			 * donc accs aux attributs de la classe FrameExample Change la
			 * couleur du texte en bleu et affiche un message
			 */
			StyleConstants.setForeground(style, Color.BLUE);

			try
			{
				document.insertString(document.getLength(), "Blue" + newline,
						style);
			}
			catch (BadLocationException ex)
			{
				System.err.println("write at bad location");
				ex.printStackTrace();
			}

			StyleConstants.setForeground(style, defaultColor);
		}
	}
}
